==> comedi-0.4.2.tgz: comedi-0.4.2/module/8255.c <==
/*
 * Driver for generic 8255
 * David A. Schleef
 */

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/malloc.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/timex.h>
#include <linux/timer.h>
#include <asm/io.h>
#include <comedi_module.h>



#define _8255_SIZE 4

#define _8255_DATA 0
#define _8255_CR 3

typedef struct{
	int cr;
}_8255_private;
#define devpriv ((_8255_private *)dev->private)

comedi_devinit _8255_init;
static int _8255_dio(comedi_device *dev,comedi_trig *ad);
static int _8255_rem(comedi_device *dev);

static int _8255_dio(comedi_device *dev,comedi_trig *it)
{
	comedi_sample s;

	switch(it->io){
	case COMEDI_IO_INPUT:
		s.data=it->data=inb(dev->iobase+_8255_DATA+it->chan);
		s.chan=it->chan;
	
		report_sample(dev,&s);
	
		return 0;
	case COMEDI_IO_OUTPUT:
		outb(it->data,dev->iobase+_8255_DATA+it->chan);
		return 0;
	default:
		return -EINVAL;
	}
}

static int _8255_sp(comedi_device *dev,comedi_param *it)
{
	int cr=devpriv->cr;
	
	switch(it->pnum){
	case COMEDI_IOBITS:
		switch(it->chan){
		case 0:
			switch(it->pval){
			case 0xff&COMEDI_IOBITS_INPUT:
				cr |= 0x10;
				break;
			case 0xff&COMEDI_IOBITS_OUTPUT:
				cr &= ~0x10;
				break;
			default:
				return -EINVAL;
			}
			break;
		case 1:
			switch(it->pval){
			case 0xff&COMEDI_IOBITS_INPUT:
				cr |= 0x02;
				break;
			case 0xff&COMEDI_IOBITS_OUTPUT:
				cr &= ~0x02;
				break;
			default:
				return -EINVAL;
			}
			break;
		case 2:
			switch(0x0f&it->pval){
			case 0x0f&COMEDI_IOBITS_INPUT:
				cr |= 0x01;
				break;
			case 0x0f&COMEDI_IOBITS_OUTPUT:
				cr &= ~0x01;
				break;
			default:
				return -EINVAL;
			}
			switch(0xf0&it->pval){
			case 0xf0&COMEDI_IOBITS_INPUT:
				cr |= 0x08;
				break;
			case 0xf0&COMEDI_IOBITS_OUTPUT:
				cr &= ~0x08;
				break;
			default:
				return -EINVAL;
			}
			break;
		default:
			return -EINVAL;
		}
		changeparam(&dev->chinfo[it->chan].paramlist,
			COMEDI_IOBITS,0,it->pval);
		outb(0x80|cr,dev->iobase+_8255_CR);
		devpriv->cr=cr;
		return 0;
	default:
		return -EINVAL;
	}
}

int _8255_init(comedi_device *dev,comedi_devconfig *it)
{
	int i;

	if(strcmp("8255",it->name))
		return 0;

	printk("comedi%d: 8255: 0x%04x ",dev->minor,it->iobase);
	if(check_region(it->iobase,_8255_SIZE)<0){
		printk("I/O port conflict\n");
		return -EIO;
	}
	request_region(it->iobase,_8255_SIZE,"8255");
	dev->iobase=it->iobase;
	dev->iosize=_8255_SIZE;
	dev->irq=0;

	dev->chan=3;

	devpriv=kmalloc(sizeof(_8255_private)+dev->chan*sizeof(comedi_chinfo),
		GFP_KERNEL);
	if(!devpriv)return -ENOMEM;
	memset(devpriv,0,sizeof(_8255_private)+dev->chan*sizeof(comedi_chinfo));

	dev->chinfo=(void *)(devpriv+1);
	for(i=0;i<3;i++){
		dev->chinfo[i].trig=_8255_dio;
		dev->chinfo[i].sp=_8255_sp;
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_BITS,COMEDI_RDONLY,8);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_AD,COMEDI_RDONLY,COMEDI_AD_DIGITAL);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_IO,COMEDI_RDONLY,COMEDI_IO_BIDIR);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_IOBITGROUP,COMEDI_RDONLY,(i==2)?4:8);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_IOBITS,COMEDI_RDWR,0xff&COMEDI_IOBITS_INPUT);
	}
	devpriv->cr=0x1b;
	outb(devpriv->cr,dev->iobase+_8255_CR);

	comedi_initbuf(&dev->rdbuf);

	dev->rem=_8255_rem;

	printk("\n");
	return 1;
}


static int _8255_rem(comedi_device *dev)
{
	int i;

	printk("comedi%d: 8255: remove\n",dev->minor);
	
	for(i=0;i<dev->chan;i++)
		delparamlist(dev->chinfo[i].paramlist);
	kfree(devpriv);
	release_region(dev->iobase,dev->iosize);

	comedi_freebuf(&dev->rdbuf);

	return 0;
}

==> comedi-0.5.4.tgz: comedi-0.5.4/module/8255.c <==
/*
 * Driver for generic 8255
 * David A. Schleef
 */

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/malloc.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/timex.h>
#include <linux/timer.h>
#include <asm/io.h>
#include <comedi_module.h>



#define _8255_SIZE 4

#define _8255_DATA 0
#define _8255_CR 3

typedef struct{
	int cr;
}_8255_private;
#define devpriv ((_8255_private *)dev->private)

comedi_devinit _8255_init;
static int _8255_dio(comedi_device *dev,comedi_trig *ad);
static int _8255_rem(comedi_device *dev);

static int _8255_dio(comedi_device *dev,comedi_trig *it)
{
	comedi_sample s;

	switch(it->io){
	case COMEDI_IO_INPUT:
		s.data=it->data=inb(dev->iobase+_8255_DATA+it->chan);
		s.chan=it->chan;
	
		return 0;
	case COMEDI_IO_OUTPUT:
		outb(it->data,dev->iobase+_8255_DATA+it->chan);
		return 0;
	default:
		return -EINVAL;
	}
}

static int _8255_sp(comedi_device *dev,comedi_param *it)
{
	int cr=devpriv->cr;
	
	switch(it->pnum){
	case COMEDI_IOBITS:
		switch(it->chan){
		case 0:
			switch(it->pval){
			case 0xff&COMEDI_IOBITS_INPUT:
				cr |= 0x10;
				break;
			case 0xff&COMEDI_IOBITS_OUTPUT:
				cr &= ~0x10;
				break;
			default:
				return -EINVAL;
			}
			break;
		case 1:
			switch(it->pval){
			case 0xff&COMEDI_IOBITS_INPUT:
				cr |= 0x02;
				break;
			case 0xff&COMEDI_IOBITS_OUTPUT:
				cr &= ~0x02;
				break;
			default:
				return -EINVAL;
			}
			break;
		case 2:
			switch(0x0f&it->pval){
			case 0x0f&COMEDI_IOBITS_INPUT:
				cr |= 0x01;
				break;
			case 0x0f&COMEDI_IOBITS_OUTPUT:
				cr &= ~0x01;
				break;
			default:
				return -EINVAL;
			}
			switch(0xf0&it->pval){
			case 0xf0&COMEDI_IOBITS_INPUT:
				cr |= 0x08;
				break;
			case 0xf0&COMEDI_IOBITS_OUTPUT:
				cr &= ~0x08;
				break;
			default:
				return -EINVAL;
			}
			break;
		default:
			return -EINVAL;
		}
		changeparam(&dev->chinfo[it->chan].paramlist,
			COMEDI_IOBITS,0,it->pval);
		outb(0x80|cr,dev->iobase+_8255_CR);
		devpriv->cr=cr;
		return 0;
	default:
		return -EINVAL;
	}
}

int _8255_init(comedi_device *dev,comedi_devconfig *it)
{
	int i;

	if(strcmp("8255",it->name))
		return 0;

	dev->iobase=it->options[0];
	printk("comedi%d: 8255: 0x%04x ",dev->minor,dev->iobase);
	if(check_region(dev->iobase,_8255_SIZE)<0){
		printk("I/O port conflict\n");
		return -EIO;
	}
	request_region(dev->iobase,_8255_SIZE,"8255");
	dev->iosize=_8255_SIZE;
	dev->driver_name="8255";
	dev->board_name="8255";
	dev->irq=0;

	dev->chan=3;

	devpriv=kmalloc(sizeof(_8255_private)+dev->chan*sizeof(comedi_chinfo),
		GFP_KERNEL);
	if(!devpriv)return -ENOMEM;
	memset(devpriv,0,sizeof(_8255_private)+dev->chan*sizeof(comedi_chinfo));

	dev->chinfo=(void *)(devpriv+1);
	for(i=0;i<3;i++){
		dev->chinfo[i].itrig=_8255_dio;
		dev->chinfo[i].sp=_8255_sp;
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_BITS,COMEDI_RDONLY,8);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_AD,COMEDI_RDONLY,COMEDI_AD_DIGITAL);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_IO,COMEDI_RDONLY,COMEDI_IO_BIDIR);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_IOBITGROUP,COMEDI_RDONLY,(i==2)?4:8);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_IOBITS,COMEDI_RDWR,0xff&COMEDI_IOBITS_INPUT);
	}
	devpriv->cr=0x1b;
	outb(devpriv->cr,dev->iobase+_8255_CR);

	comedi_initbuf(&dev->rdbuf);

	dev->rem=_8255_rem;

	printk("\n");
	return 1;
}


static int _8255_rem(comedi_device *dev)
{
	int i;

	printk("comedi%d: 8255: remove\n",dev->minor);
	
	for(i=0;i<dev->chan;i++)
		delparamlist(dev->chinfo[i].paramlist);
	kfree(devpriv);
	release_region(dev->iobase,dev->iosize);

	comedi_freebuf(&dev->rdbuf);

	return 0;
}
